import os
import subprocess
import sys
from datetime import datetime
from pathlib import Path

from jinja2 import Environment, PackageLoader, select_autoescape


def get_template(template_name: str = "plan.html", plan_path: Path | None = None):
    """ """
    templates = Environment(
        loader=PackageLoader("plan"), autoescape=select_autoescape()
    )
    template = templates.get_template(template_name)
    today = datetime.today().strftime("%Y-%m-%d")
    return template.render(today=today, plan_path=plan_path)


def plan_path() -> Path:
    """
    returns: Path to current date's .plan.md
    """
    today = datetime.today().strftime("%Y-%m-%d")
    path = Path().home() / ".plan" / f"{today}.md"
    return path


def print_plan(plan_path: Path):
    """
    send .plan.md to printer.

    uses pandoc and lp shell tools.
    """
    # preamble ="""
    #     ---
    #     documentclass: extarticle
    #     fontsize: 20pt
    #     ---
    # """
    cat_process = subprocess.Popen(["cat", str(plan_path)], stdout=subprocess.PIPE)
    pandoc_process = subprocess.Popen(
        ["pandoc", "-t", "pdf", "-V", "geometry:margin=.5in", "-"],
        stdin=cat_process.stdout,
        stdout=subprocess.PIPE,
    )

    subprocess.run(
        ["lp", "-o", "sides=two-sided-long-edge", "-o", "number-up=2"],
        stdin=pandoc_process.stdout,
    )


def create_and_open_template(plan: Path, name: str = "plan.html") -> int:
    plan.parent.mkdir(parents=True, exist_ok=True)
    plan.touch()

    template = get_template(name, plan_path=plan)
    with plan.open("w") as f:
        f.write(template)

    editor = os.getenv("EDITOR", "nvim")
    os.chdir(plan.parent)
    p = subprocess.Popen([editor, str(plan)])
    sys.exit(p.wait())


def open_in_editor(plan: Path):
    plan.parent.mkdir(parents=True, exist_ok=True)
    if not plan.exists():
        plan.touch()
        with plan.open("w") as f:
            f.write(default_template(plan_path=plan))
    editor = os.getenv("EDITOR", "nvim")
    os.chdir(Path().home() / ".plan")
    subprocess.run([editor, str(plan)])
