import sys
from pathlib import Path

import click
from click_default_group import DefaultGroup

from .main import (
    create_plan,
    edit_plan,
    plan_path,
    print_plan,
)


@click.group(cls=DefaultGroup, default="main", default_if_no_args=True)
def cli():
    pass


@cli.command("print")
@click.option(
    "--plan_path",
    "-p",
    "plan_path",
    type=click.Path(exists=True, dir_okay=False, readable=True, path_type=Path),
    default=plan_path(),
)
def send_to_printer(plan_path: Path):
    print_plan(plan_path)


@cli.command("main")
@click.option(
    "-t",
    "--template",
    "template",
    required=True,
    type=click.Choice(["plan.html", "work.html"]),
    prompt=True,
    default="plan.html",
    show_default=True,
)
@click.option(
    "--print/--no-print",
    "should_print",
    is_flag=True,
    default=False,
    prompt="print template only?",
    required=True,
)
def open_today(template, should_print):
    """open today"""
    path = plan_path()
    content = create_plan(plan_path=path, name=template)
    if should_print:
        print(content)
    else:
        sys.exit(edit_plan(path))
