import os
import subprocess
import sys
from datetime import datetime
from pathlib import Path

from jinja2 import Environment, PackageLoader, select_autoescape


def get_template(template_name: str = "plan.html", plan_path: Path | None = None):
    """ """
    templates = Environment(
        loader=PackageLoader("plan"), autoescape=select_autoescape()
    )
    breakpoint()
    template = templates.get_template(template_name)
    today = datetime.today().strftime("%Y-%m-%d")
    return template.render(today=today, plan_path=plan_path)


def plan_path(date: datetime | None = None) -> Path:
    """
    returns: Path to current date's .plan.md
    """
    if date is None:
        date = datetime.today()
    formatted = date.strftime("%Y-%m-%d")
    path = Path().home() / ".plan" / f"{formatted}.md"
    return path


def print_plan(plan_path: Path):
    """
    send .plan.md to printer.

    uses pandoc and lp shell tools.
    """
    preamble = """
        ---
        documentclass: extarticle
        fontsize: 20pt
        ---
    """
    preamble = subprocess.Popen(["echo", "-n", preamble], stdout=subprocess.PIPE)
    cat = subprocess.Popen(
        ["cat", "-", str(plan_path)],
        stdin=preamble.stdout,
        stdout=subprocess.PIPE,
    )
    pandoc = subprocess.Popen(
        ["pandoc", "-t", "pdf", "-V", "geometry:margin=.5in", "-"],
        stdin=cat.stdout,
        stdout=subprocess.PIPE,
    )
    send = subprocess.Popen(
        ["lp", "-o", "sides=two-sided-long-edge", "-o", "number-up=2"],
        stdin=pandoc.stdout,
    )
    sys.exit(send.wait())


def create_plan(plan_path: Path, name: str = "plan.html", force=False) -> str:
    plan_path.parent.mkdir(parents=True, exist_ok=True)

    if not plan_path.exists() or force:
        content = get_template(name, plan_path=plan_path)
        with plan_path.open("w") as f:
            f.write(content)
    else:
        content = get_plan(plan_path)
    return content


def get_plan(plan_path: Path) -> str:
    with plan_path.open("r") as f:
        content = f.read()
    return content


def edit_plan(plan_path: Path) -> int:
    editor = os.getenv("EDITOR", "nvim")
    os.chdir(plan_path.parent)
    p = subprocess.Popen([editor, str(plan_path)])
    return p.wait()
